package org.softevo.ample;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.softevo.ample.instrument.InstrumentationConfiguration;
import org.softevo.ample.ui.IAmplePreferenceConstants;
import org.softevo.ample.ui.views.RankingView;

/**
 * Main plugin class.
 */
public class AmplePlugin extends AbstractUIPlugin implements IAmplePreferenceConstants { 
	
  public static final boolean DEBUG = false;
  
  /** The ranking view that displays the results. <BR>
   */
  private static RankingView rankingView = null;
  
  /** Gets the ranking view. <BR>
   */
  public static RankingView getRankingView() {
    return rankingView;
  }

  /** Sets the ranking view. <BR>
   */
  public static void setRankingView(RankingView rankingView) {
    AmplePlugin.rankingView = rankingView;
  }

  /**
	 * Initializes a new instance of <code>AmplePlugin</code>.
	 */
	public AmplePlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("org.softevo.ample.AmplePluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		listener = null;
	}

	/**
	 * This method is called upon plug-in activation.
	 * {@inheritDoc}
	 */
	public void start(BundleContext context) throws Exception {
	  System.out.println("Ample Plugin: Starting AMPLE.");
	  super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped.
	 * {@inheritDoc}
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Gets the shared instance.
	 * 
	 * @return The shared instance of <code>AmplePlugin</code>.
	 */
	public static AmplePlugin getDefault() {
		return plugin;
	}

	/**
	 * Gets the string from the plugin's resource bundle, or 'key' if not found.
	 * 
	 * @param key Key of string to retrieve from the resource bundle.
	 * @return The string that is mapped to the specified key; the key itself if it is not found in
	 *         the resource bundle.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = AmplePlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Gets the shared JUnit test run listener.
	 * 
	 * @return The shared instance of <code>AmpleTestRunListener</code>; <code>null</code> if
	 *         still none is activated.
	 * @see #getListener()
	 */
	public static AmpleTestRunListener getSharedListener() {
		return getDefault().listener;
	}

	/**
	 * Gets the plugin's resource bundle.
	 * 
	 * @return The resource bundle of the plugin.
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * Gets the shared JUnit test run listener.
	 * 
	 * @return The shared instance of <code>AmpleTestRunListener</code>; <code>null</code> if
	 *         still none is activated.
	 * @see #getSharedListener()
	 */
	public AmpleTestRunListener getListener() {
		return listener;
	}

	/**
	 * Sets the shared JUnit test run listener.
	 * 
	 * @param aListener Instance of currently activate <code>AmpleTestRunListener</code>.
	 */
	public void setListener(AmpleTestRunListener aListener) {
		listener = aListener;
	}

	/**
	 * The shared instance.
	 */
	private static AmplePlugin plugin;

	/**
	 * Resource bundle of the plugin.
	 */
	private ResourceBundle resourceBundle;

	/**
	 * Ample's JUnit test run listener.
	 */
	private AmpleTestRunListener listener;
    
	/** Gets an instrumentation configuration with the values currently set in the
     *  preference store. <BR>
	 */
    public InstrumentationConfiguration getInstrumentationConfiguration() {
      return new InstrumentationConfiguration(getPreferenceStore().getInt(KEY_WINDOWSIZE),
                                              getPreferenceStore().getInt(KEY_LEVEL), 
                                              getPreferenceStore().getInt(KEY_PERSPECTIVE));
    }
    
    /** Gets the maximum number of seconds to wait for a test to finish. <BR>
     */
    public int getMaxTestDurationSeconds() {
      return (getPreferenceStore().getInt(KEY_TESTDURATION) + 1) * 60;
    }
}
