package org.softevo.ample;

import java.util.ArrayList;

import org.eclipse.jdt.internal.junit.ui.TestRunInfo;
import org.eclipse.jdt.junit.ITestRunListener;

/**
 * JUnit test run listener.
 * <p>The instance of this class maintains up-to-date lists of passing and failing tests.</p>
 * <p>The Ample plugin initializes and registers a single instance of this class. It can be accessed
 * by calling the method {@see org.softevo.ample.AmplePlugin#getSharedListener()}.
 */
public class AmpleTestRunListener implements ITestRunListener
{
	// TODO: Implement ITestRunListener3 when the interface is documented

	/**
	 * Initializes a new instance of <code>AmpleTestRunListener</code>.
	 * <p>Note: This class is not intended to be instatiated by clients.</p>
	 */
	public AmpleTestRunListener() {
		testsAreRunning = false;
		passingTests = new ArrayList(); 
		failingTests = new ArrayList();
		AmplePlugin.getDefault().setListener(this);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener3#testFailed(int, String, String, String, String,
	 *      String)
	 */
	public void testFailed(int status, String testId, String testName, String trace,
			String expected, String actual) {
  //		System.out.print("PopupMenuAction.testFailed(" + testId + ") ");

		// Add the test to the list of failed tests
		TestRunInfo test = addTestIfNeeded(failingTests, testId, testName);
		test.setActual(actual);
		test.setExpected(expected);
		test.setStatus(status);
		test.setTrace(trace);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener3#testReran(String, String, String, int, String,
	 *      String, String)
	 *
	public void testReran(String testId, String className, String testName, int statusCode,
			String trace, String expectedResult, String actualResult) {
		System.out.println("PopupMenuAction.testReran(2)");
		// Implement method when this class is updated to implement ITestRunListener3
	}*/

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener2#testTreeEntry(String)
	 *
	public void testTreeEntry(String description) {
		System.out.println("PopupMenuAction.testTreeEntry()");
		// Implement method when this class is updated to implement ITestRunListener2
	}*/

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testRunStarted(int)
	 */
	public void testRunStarted(int testCount) {
//		System.out.println("PopupMenuAction.testRunStarted()");
		testsAreRunning = true;
		passingTests.clear();
		failingTests.clear();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testRunEnded(long)
	 */
	public void testRunEnded(long elapsedTime) {
//		System.out.println("PopupMenuAction.testRunEnded()");
		testsAreRunning = false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testRunStopped(long)
	 */
	public void testRunStopped(long elapsedTime) {
//		System.out.println("PopupMenuAction.testRunStopped()");
		testsAreRunning = false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testRunTerminated()
	 */
	public void testRunTerminated() {
//		System.out.println("PopupMenuAction.testRunTerminated()");
		testsAreRunning = false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testStarted(String, String)
	 */
	public void testStarted(String testId, String testName) {
//		System.out.println("PopupMenuAction.testStarted()");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testEnded(String, String)
	 */
	public void testEnded(String testId, String testName) {
//		System.out.println("PopupMenuAction.testEnded(" + testId + ")");

		if (!testHasFailed(testId))
			if (!failingTests.contains(testId)) {
				TestRunInfo tInfo = new TestRunInfo(testId, testName);
				passingTests.add(tInfo);
			}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testFailed(int, String, String, String)
	 */
	public void testFailed(int status, String testId, String testName, String trace) {
//		System.out.println("PopupMenuAction.testFailed(1)");
		testFailed(status, testId, testName, trace, null, null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.junit.ITestRunListener#testReran(String, String, String, int, String)
	 */
	public void testReran(String testId, String testClass, String testName, int status, String trace) {
//		System.out.println("PopupMenuAction.testReran()");
		// TODO: See if testClass can be used; currently this argument is ignored
		TestRunInfo tInfo;
		switch (status) {
			case ITestRunListener.STATUS_OK:
				removeTest(failingTests, testId);
				tInfo = addTestIfNeeded(passingTests, testId, testName);
				break;
			default:
				removeTest(passingTests, testId);
				tInfo = addTestIfNeeded(failingTests, testId, testName);
				break;
		}
		tInfo.setStatus(status);
		tInfo.setTrace(trace);
	}

	/**
	 * Checks if there are tests currently running.
	 * 
	 * @return <code>true</code> if JUnit tests are currently running; <code>false</code>
	 *         otherwise.
	 */
	public boolean areTestsRunning() {
		return testsAreRunning;
	}

	/**
	 * Gets the number of passing tests.
	 * 
	 * @return Number of of passing tests present.
	 */
	public int getPassingTestsCount() {
		return passingTests.size();
	}

	/**
	 * Gets the list of passing tests.
	 * 
	 * @return <code>ArrayList</code> of all the passing tests.
	 */
	public ArrayList getPassingTests() {
		return passingTests;
	}

	/**
	 * Gets an array of all the specified passing tests.
	 * 
	 * @param indices Array of zero-based indices of passing tests in the list of passing tests.
	 * @return Array of <code>TestRunInfo</code> instances corresponding to the given indices.
	 * @throws IndexOutOfBoundException When any of the specified indices is out of range.
	 */
	public TestRunInfo[] getPassingTests(int[] indices) {
		TestRunInfo[] tests = new TestRunInfo[indices.length];
		for (int i = 0; i < indices.length; ++i) {
			tests[i] = getPassingTest(indices[i]);
		}
		return tests;
	}

	/**
	 * Gets the <code>TestRunInfo</code> at the specified position in the list of passing tests.
	 * <p>
	 * This method is used for simplifying the call:<br/>
	 * <code>(TestRunInfo) AmplePlugin.getSharedListener().getPassingTests().get(i)</code><br/>
	 * to:<br/>
	 * <code>AmplePlugin.getSharedListener().getPassingTest(i)</code>
	 * </p>
	 * 
	 * @param index Zero-based index of the test to return.
	 * @return Information on the test at the specified index in the list of passing tests.
	 * @throws IndexOutOfBoundException When specified index is out of range.
	 */
	public TestRunInfo getPassingTest(int index) {
		return (TestRunInfo) passingTests.get(index);
	}

	/**
	 * Gets the list of failing tests.
	 * 
	 * @return <code>ArrayList</code> of all the failing tests.
	 */
	public ArrayList getFailingTests() {
		return failingTests;
	}

	/**
	 * Gets the <code>TestRunInfo</code> at the specified position in the list of failing tests.
	 * <p>
	 * This method is used for simplifying the call:<br/>
	 * <code>(TestRunInfo) AmplePlugin.getSharedListener().getFailingTests().get(i)</code><br/>
	 * to:<br/>
	 * <code>AmplePlugin.getSharedListener().getFailingTest(i)</code>
	 * </p>
	 * 
	 * @param index Zero-based index of the test to return.
	 * @return Information on the test at the specified index in the list of failing tests.
	 * @throws IndexOutOfBoundException When specified index is out of range.
	 */
	public TestRunInfo getFailingTest(int index) {
		return (TestRunInfo) failingTests.get(index);
	}

	/**
	 * Gets the index of the specified failing test.
	 * 
	 * @param testInfo Information of the failing test the get the index of.
	 * @return Zero-based index of the given failing test, <code>-1</code> if the specified test
	 *         does not belong to the list of failing tests.
	 * @see #getFailingTests()
	 */
	public int indexOfFailing(TestRunInfo testInfo) {
		String testId = testInfo.getTestId();
		for (int i = 0; i < failingTests.size(); ++i) {
			if (((TestRunInfo) failingTests.get(i)).getTestId().equals(testId)) {
				return i;
			}
		}
		return -1;
	}

    /** Utility method that searches a list of runs for a run with
     *  a given id. <BR>
     *  @param runs the list of runs to be searched
     *  @param testId the id of the sought teset
     *  @return the info object or <code>null</code> if nothing was found
     */
    public static TestRunInfo findTestRun(ArrayList runs, String testId) {
      TestRunInfo info;
      
      for (int counter = 0; counter < runs.size(); counter++) {
        info = (TestRunInfo) runs.get(counter);
        if (info.getTestId().equals(testId)) {
          return info;
        }
      }
      return null;
    }

    /** Utility method that searches a failing test with
     *  a given id. <BR>
     *  @param testId the id of the sought teset
     *  @return the info object or <code>null</code> if nothing was found
     */
    public TestRunInfo findFailingTestRun(String testId) {
      return findTestRun(failingTests, testId);
    }

    /** Utility method that searches a passing test with
     *  a given id. <BR>
     *  @param testId the id of the sought teset
     *  @return the info object or <code>null</code> if nothing was found
     */
    public TestRunInfo findPassingTestRun(String testId) {
      return findTestRun(passingTests, testId);
    }

	/**
	 * Checks if a specified test has failed.
	 * 
	 * @param aTestId ID of test to verify.
	 * @return <code>true</code> if {@link #failingTests} contains a test with the specified ID;
	 *         <code>false</code> otherwise.
	 */
	private boolean testHasFailed(String aTestId) {
		for (int i = 0; i < failingTests.size(); ++i) {
			TestRunInfo tInfo = (TestRunInfo) failingTests.get(i);
			if (tInfo.getTestId().equals(aTestId)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Removes a test, if present, from a speicified list of tests.
	 * 
	 * @param aTests List of tests to be updated.
	 * @param aTestId ID of test to be removed.
	 */
	private void removeTest(ArrayList aTests, String aTestId) {
		for (int i = 0; i < aTests.size(); ++i) {
			TestRunInfo tInfo = (TestRunInfo) aTests.get(i);
			if (tInfo.getTestId().equals(aTestId)) {
				aTests.remove(i);
				return;
			}
		}
	}
	
	/**
	 * Adds a test to a given list of tests if not already present.
	 * 
	 * @param aTests List of test to search in.
	 * @param aTestId ID of the test to be added.
	 * @param aTestName Name of the test to be added.
	 * @return Test with the specified ID as found in (or added to) the given test list.
	 */
	private TestRunInfo addTestIfNeeded(ArrayList aTests, String aTestId, String aTestName) {
		TestRunInfo tInfo = null;
		for (int i = 0; i < aTests.size(); ++i) {
			TestRunInfo info = (TestRunInfo) aTests.get(i);
			if (info.getTestId().equals(aTestId)) {
				// Test already present in the list
				tInfo = info;
				break;
			}
		}

		if (tInfo == null) {
			// Test not found, add it to the list
			tInfo = new TestRunInfo(aTestId, aTestName);
			aTests.add(tInfo);
		}
		return tInfo;
	}

	/**
	 * Flag indictating that tests are currently running.
	 */
	private volatile boolean testsAreRunning;

	/**
	 * List of JUnit tests that have passed.
	 */
	private ArrayList passingTests;

	/**
	 * List of JUnit tests that have failed in assertions or due to exceptions.
	 */
	private ArrayList failingTests;
}
