/*
 *
 */
package org.softevo.ample.eval;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

public class AverageSequenceWeightEvaluator {

  private XMLPatternFileReader failingTestPatternsReader;
  
  private Vector passingTestPatternsReaderList;
  
  private HashMap classNameToPatternDifferencesMap = null;
  
  public AverageSequenceWeightEvaluator(XMLPatternFileReader failingTestPatterns,
                                        Vector passingTestPatterns) {
    this.failingTestPatternsReader = failingTestPatterns;
    this.passingTestPatternsReaderList = passingTestPatterns;
    classNameToPatternDifferencesMap = new HashMap();
  }
  
  protected double getAverageSequenceWeight(String className, HashSet failingPatterns, Vector passingPatternsList) {
    HashSet passingPatterns;
    Pattern pattern;
    HashSet union;
    Iterator patternIterator;
    double patternWeight = 0, sumPatternWeights = 0;
    int failingTestCounter = 0, passingTestCounter = 0;
    PatternDifferences differences;
    
    union = new HashSet();
    differences = new PatternDifferences(className);
    patternIterator = failingPatterns.iterator();
    while (patternIterator.hasNext()) {
      union.add(patternIterator.next());
    }
    for (int counter = 0; counter < passingPatternsList.size(); counter++) {
      passingPatterns = (HashSet) passingPatternsList.get(counter);
      patternIterator = passingPatterns.iterator();
      while (patternIterator.hasNext()) {
        union.add(patternIterator.next());
      }
    }
    patternIterator = union.iterator();
    while (patternIterator.hasNext()) {
      failingTestCounter = 0;
      passingTestCounter = 0;
      pattern = (Pattern) patternIterator.next();
      if (failingPatterns.contains(pattern)) {
        failingTestCounter++;
      }
      for (int counter = 0; counter < passingPatternsList.size(); counter++) {
        passingPatterns = (HashSet) passingPatternsList.get(counter);
        if (passingPatterns.contains(pattern)) {
          passingTestCounter++;
        }
      }
      patternWeight = Math.abs((double)((double) failingTestCounter - ((double) passingTestCounter/(double) passingTestPatternsReaderList.size())));
      sumPatternWeights += patternWeight;
      differences.addPatternAndWeight(new PatternAndWeight(pattern, patternWeight, failingTestCounter, passingTestCounter));
    }
    classNameToPatternDifferencesMap.put(className, differences);
    return (double) (sumPatternWeights/(double)union.size());
  }
  
  public ClassNameWithWeight[] calculateRanking() {
    Vector ranking;
    Iterator classNameIterator;
    String className;
    double averageSequenceWeight;
    Vector passingPatternSets;
    XMLPatternFileReader passingTestReader;
    
    ranking = new Vector();
    classNameIterator = failingTestPatternsReader.getClassToPatternSetMap().keySet().iterator();
    while (classNameIterator.hasNext()) {
      className = classNameIterator.next().toString();
      passingPatternSets = new Vector(passingTestPatternsReaderList.size());
      for (int counter = 0; counter < passingTestPatternsReaderList.size(); counter++) {
        passingTestReader = (XMLPatternFileReader) passingTestPatternsReaderList.get(counter);
        if (passingTestReader.getClassToPatternSetMap().containsKey(className)) {
          passingPatternSets.add(passingTestReader.getClassToPatternSetMap().get(className));
        }
      }
      averageSequenceWeight = getAverageSequenceWeight(className, 
                                                       (HashSet) failingTestPatternsReader.getClassToPatternSetMap().get(className),
                                                       passingPatternSets);
      ranking.add(new ClassNameWithWeight(className, averageSequenceWeight));
      
    }
    Collections.sort(ranking);
    return (ClassNameWithWeight[]) ranking.toArray(new ClassNameWithWeight[ranking.size()]);
  }

  public HashMap getClassNameToPatternDifferencesMap() {
    return classNameToPatternDifferencesMap;
  }
}
