/**
 * <p>Title: Bytecode instrumentation</p>
 * <p>Description: Tracing framework for java bytecode</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universitt Saarbrcken</p>
 * @author Valentin Dallmeier
 * @version 1.0
 */
package org.softevo.ample.eval;

import java.security.MessageDigest;

/** A pattern and the index in the trace it was discovered. <BR>
 */
public class Pattern implements Comparable {

  /** The message digest for md5 hashing. <BR>
   */
  public static MessageDigest digest = null;

  /** The sequence of method calls. <BR>
   */
  private String[] methodCallSequence = null;

  /** The discovery index of the pattern. <BR>
   */
  private int discoveryIndex = 0;

  /** The number of times the pattern occured. <BR>
   */
  private int numberOfOccurences = 0;
  
  /** The last time this pattern was found. <BR>
   */
  private int lastOccurence = 0;
  
  /** Creates a new pattern. <BR
   *  @param methodCalls the call sequence
   *  @param discoveryIndex the index in the trace the pattern was discovered
   */
  public Pattern(String[] methodCallSequence,
                 int discoveryIndex) {
    this.methodCallSequence = methodCallSequence;
    this.discoveryIndex = discoveryIndex;
    try {
      digest = MessageDigest.getInstance("MD5");
    } catch (Exception exception) {
      //pray this never occurs
    }	
  }		

  /** Returns a textual representation of the method call
   *  sequence. <BR>
   */
  public String toString() {
    StringBuffer result;
    
    result = new StringBuffer();
    for (int counter = 0; counter < methodCallSequence.length; counter++) {
      result.append(methodCallSequence[counter]);
      result.append('#');
    }
    return result.toString();
  }
  
  /** Increases the occurence counter by one. <BRO>
   */
  public void countOccurence() {
    numberOfOccurences++;
  }

  /** Gets the number of times the pattern was found. <BR>
   */
  public int getNumberOfOccurences() {
    return numberOfOccurences;
  }

  /** Gets the method call sequence. <BR>
   */
  public String[] getMethodCallSequence() {
    return methodCallSequence;
  }

  /** Gets the discovery index for the pattern. <BR>
   */
  public int getDiscoveryIndex() {
    return discoveryIndex;
  }

  /** Gets the hash code of the concatenated method
   *  calls. <BR>
   */
  public int hashCode() {
    StringBuffer buffer;

    buffer = new StringBuffer(1000);
    for (int counter = 0; counter < methodCallSequence.length; counter++) {
      buffer.append(methodCallSequence[counter]);
    }
    return buffer.toString().hashCode();
  }

  /** Compares this object by the number of references. <BR>
   *  @param object the object to compare this entry to
   */
  public int compareTo(Object object) {
    return discoveryIndex - ((Pattern) object).discoveryIndex;
  }

  /** Two patterns are considered equal if they
   *  have the same method call sequence. <BR>
   *  @param object another pattern
   */
  public boolean equals(Object object) {
    Pattern pattern;

    if ((object != null) && (object instanceof Pattern)) {
      pattern = (Pattern) object;
      if (pattern.methodCallSequence.length != methodCallSequence.length) {
        return false;
      }
      for (int counter = 0; counter < methodCallSequence.length; counter++) {
        if (!methodCallSequence[counter].equals(pattern.methodCallSequence[counter])) {
          return false;
        }
      }
      return true;
    } else {
      return false;
    }
  }
  
  /** Gets a md5 hash of the canonical pattern. <BR>
   */
  public byte[] getHash() {
    return digest.digest(getCanonicalPattern().getBytes());
  }

  /** Gets the last occurence of this pattern. <BR>
   */
  public int getLastOccurence() {
    return lastOccurence;
  }

  /** Sets a new value for the last occurence. <BR>
   */
  public void setLastOccurence(int lastOccurence) {
    this.lastOccurence = lastOccurence;
  }
  
  /** Sets the number of occurences. <BR>
   */
  public void setOccurences(int occurences) {
    this.numberOfOccurences = occurences;
  }
  
  /** Gets the canonical form of the pattern. <BR>
   *  This method simply concatenates the method names using
   *  the separator defined in <code>SimpleFormat</code>.
   *  @return the canonical pattern
   */
  public String getCanonicalPattern() {
    StringBuffer result;
    
    result = new StringBuffer();
    for (int counter = 0; counter < methodCallSequence.length; counter++) {
      result.append(methodCallSequence[counter]);
      if (counter != methodCallSequence.length - 1) {
        result.append("#");
      }
    }
    return result.toString();
  }

}
