/*
 *
 */
package org.softevo.ample.eval;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author dallmeier
 *
 */
public class XMLPatternFileReader {

  /** The result of the parse. <BR>
   */
  protected HashMap classToPatternSetMap = null;
  
  protected HashMap classToMethodsMap = null;
  
  /** The name of the xml pattern file. <BR>
   */
  private String fileName = null;

  /** The total number of trace entries. <BR>
   */
  private int traceLength = -1;

  /** The window size of the run. <BR>
   */
  private int windowSize = -1;

  /** Creates a new reader for a pattern file. <BR>
   *  @param fileName the name of the file
   */
  public XMLPatternFileReader(String fileName) {
    //support for large xml files
    System.setProperty("entityExpansionLimit", "10000000");
    this.fileName = fileName;
    classToPatternSetMap = new HashMap();
    classToMethodsMap = new HashMap();
  }

  public HashMap getCalledMethodsMap() {
    return classToMethodsMap;
  }
  
  public static void main(String[] argv) throws Exception {
    XMLPatternFileReader reader;
    HashMap theSet;
    Iterator scanner;
    Vector patterns;
    Object key;
    
    reader = new XMLPatternFileReader("/home/dallmeier/dipl/test/jakarta-tomcat-5.0.25-src/patterns.xml");
    reader.parseFile();
    theSet = reader.classToPatternSetMap;
    scanner = theSet.keySet().iterator();
    while (scanner.hasNext()) {
      key = scanner.next();
      System.out.println(key);
      patterns = (Vector) theSet.get(key);
      for (int counter = 0; counter < patterns.size(); counter++) {
        System.out.println(((Pattern) patterns.get(counter)).getCanonicalPattern());
      }
    }
  }
  
  public void parseFile() throws AnalysisException {
    SAXParserFactory parserFactory;
    SAXParser parser;
    FileInputStream fileInputStream = null;
    final File traceFile;
    
    parserFactory = SAXParserFactory.newInstance();
    try {
      parser = parserFactory.newSAXParser();
      traceFile = new File(fileName);
      fileInputStream = new FileInputStream(traceFile);
      parser.parse(fileInputStream,
                   new ParseHandler());
    } catch (Exception parserException) {
      throw new AnalysisException(parserException);
    } finally {
      if (fileInputStream != null) {
        try {
          fileInputStream.close();
        } catch (IOException finalizeException) {
          //ignore exception
        }
      }
    }
  }

  /** Gets the patterns read. <BR>
   */
  public HashMap getClassToPatternSetMap() {
    return classToPatternSetMap;
  }
  
  /** Gets the window size. <BR>
   */
  public int getWindowSize() {
    return windowSize;
  }
  
  /** Gets the trace length. <BR>
   */
  public int getTraceLength() {
    return traceLength;
  }
  
  /** The handler used to parse the file. <BR>
   */
  protected class ParseHandler extends DefaultHandler {

    /** String constant for pattern element. <BR>
     */
    public static final String EL_PATTERNS = "patterns";
    
    /** String constant for class element. <BR>
     */
    public static final String EL_CLASS = "class";
    
    /** String constant for pattern element. <BR>
     */
    public static final String EL_PATTERN = "pattern";
    
    /** String constant for method element. <BR>
     */
    public static final String EL_METHOD = "method";

    /** String constant for symbols element. <BR>
     */
    public static final String EL_SYMBOLS = "symbols";
    
    /** The class name currently processed. <BR>
     */
    protected String className = null;
    
    /** The methods. <BR>
     */
    protected Vector methods = null;
    
    /** The patterns for the class currently analysed. <BR>
     */
    protected HashSet patterns = null;
    
    /** The discovery index of the pattern currently analysed. <BR>
     */
    protected int discoveryIndex = -1;
    
    /** The number of times the pattern occured. <BR>
     */
    protected int occurences = -1;

    /** The last time the pattern occured. <BR>
     */
    protected int lastOccurence = -1;
    
    protected boolean parsingCalledSymbolsElement = false;
    
    protected HashMap methodToOccurencesMap = null;
    
    /** Sets the state and the element type
     *  depending on the name of the element. <BR>
     *  @param namespaceURI the active namespace
     *  @param localName the unqualified element name
     *  @param qName the qualified element name
     *  @param atts the attributes of the element
     */
    public void startElement(String namespaceURI,
                             String localName,
                             String qName,
                             Attributes atts) {
      if (qName.equals(EL_PATTERNS)) {
        traceLength = Integer.parseInt(atts.getValue("traceSize"));
        windowSize = Integer.parseInt(atts.getValue("windowSize"));
      } else if (qName.equals(EL_SYMBOLS)) {
        parsingCalledSymbolsElement = true;
      } else if (qName.equals(EL_CLASS)) {
        if (!parsingCalledSymbolsElement) {
          className = atts.getValue("name");
          patterns = new HashSet();
        } else {
          methodToOccurencesMap = new HashMap();
          classToMethodsMap.put(atts.getValue("name"), methodToOccurencesMap);
        }
      } else if (qName.equals(EL_PATTERN)) {
        methods = new Vector();
        discoveryIndex = Integer.parseInt(atts.getValue("discoveryIndex"));
        occurences = Integer.parseInt(atts.getValue("occurences"));
        lastOccurence = Integer.parseInt(atts.getValue("lastOccurence"));
      } else if (qName.equals(EL_METHOD)) {
        if (!parsingCalledSymbolsElement) {
          methods.add(atts.getValue("name"));
        } else {
          methodToOccurencesMap.put(atts.getValue("name"), new Integer(atts.getValue("calls")));
        }
      } else {
        System.out.println("Found unknown element " + qName +
                           " while parsing.");
      }
    }

    /** Propagates the entry to all associated
     *  analysers. <BR>
     */
    public void endElement(String namespaceURI,
                           String localName,
                           String qName) {
      Pattern pattern;
      
      if (qName.equals(EL_PATTERNS)) {
        //do nothing
      } else if (qName.equals(EL_SYMBOLS)) {
        parsingCalledSymbolsElement = false;
      } else if (qName.equals(EL_CLASS)) {
        if (!parsingCalledSymbolsElement) {
          classToPatternSetMap.put(className, patterns);
        }
      } else if (qName.equals(EL_PATTERN)) {
        pattern = new Pattern(((String[]) methods.toArray(new String[methods.size()])),
                              discoveryIndex);
        pattern.setLastOccurence(lastOccurence);
        pattern.setOccurences(occurences);
        patterns.add(pattern);
      } else if (qName.equals(EL_METHOD)) {
        //do nothing
      }
    }

    /** This method provides default exception handling
     *  (currently a call to <code>System.exit()</code>. <BR>
     *  @param exception the exception that occured
     */
    protected void handleException(AnalysisException exception) {
      exception.printStackTrace();
      System.exit( -1);
    }
  }

}
