/*
 *
 */
package org.softevo.ample.instrument;

import org.softevo.ample.ui.IAmplePreferenceConstants;

/** Configuration object that holds properties of the instrumentation. <BR>
 * @author dallmeier
 */
public class InstrumentationConfiguration implements IAmplePreferenceConstants {

  /** The window size to use. <BR>
   */
  private int windowSize = 0;
  
  /** The trace level to be used. <BR>
   */
  private int level = 0;
  
  /** Indicates if calls are associated with the
   *  caller (outgoing calls) or the callee (incoming calls).
   */
  private int callAssociation = 0;
  
  /** The name of the output file. <BR>
   */
  private String outputFileName = null;
  
  /** The name of the identity map file. <BR>
   */
  private String identityMapFileName = null;
  
  /** Creates a new configuration object. <BR>
   *  @param windowSize the window size
   *  @param level the level to trace on.
   *  @param callAssociation what calls are associated with
   *  @param outputFileName the output file
   *  @param identityMapFileName the name of the identity map file
   */
  public InstrumentationConfiguration(int windowSize, int level, 
                                      int callAssociation) {
    this.windowSize = windowSize;
    this.level = level;
    this.callAssociation = callAssociation;
    this.outputFileName = outputFileName;
    this.identityMapFileName = identityMapFileName;
  }

  /** Gets the window size. <BR>
   */
  public int getWindowSize() {
    return Integer.parseInt(PREF_WINDOWSIZES[windowSize]);
  }
  
  /** Gets the trace level. <BR>
   */
  public int getLevel() {
    return level;
  }

  /** Gets the call association from the config. <BR>
   */
  public int getCallAssociation() {
    return callAssociation;
  }
  
  /** Tests if incoming calls should be traced. <BR>
   */
  public boolean isIncomingCalls() {
    return callAssociation == CALLS_INCOMING;
  }
  
  /** Tests if outgoing calls should be traced. <BR>
   */
  public boolean isOutgoingCalls() {
    return callAssociation == CALLS_OUTGOING;
  }
  
  
  /** Tests if tracing shall be done on object level. <BR>
   */
  public boolean isObjectLevel() {
    return level == LEVEL_OBJECT;
  }
  
  /** Tests if tracing shall be done on class leve. <BR>
   *  @return
   */
  public boolean isClassLevel() {
    return level == LEVEL_CLASS;
  }

  /** Gets the name of the output file. <BR>
   *  @return
   */
  public String getOutputFileName() {
    return outputFileName;
  }

  /** Gets the name of the identity map file. <BR>
   *  @return
   */
  public String getIdentityMapFileName() {
    return identityMapFileName;
  }

  /** Sets the name of the identity map file. <BR>
   */
  public void setIdentityMapFileName(String identityMapFileName) {
    this.identityMapFileName = identityMapFileName;
  }

  /** Sets the name of the output file. <BR>
   */
  public void setOutputFileName(String outputFileName) {
    this.outputFileName = outputFileName;
  }
  
  public String toString() {
    return PREF_PERSPECTIVES[callAssociation] + " calls on the " + PREF_LEVELS[level] + " level for window size " + getWindowSize();
  }
}
