/*
 *
 */
package org.softevo.ample.instrument;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.FieldGen;
import org.apache.bcel.generic.InstructionConstants;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.PUSH;
import org.apache.bcel.generic.Type;

/** This class creates bytecode for the <code>Options</code> class. <BR> 
 *  The purpose of creating this class is to provide a static initializer
 *  that pushes the correct initialization values into the properties map.
 *  @author dallmeier
 *
 */
public class OptionsClassCreator implements Constants {

  /** A factory for new instructions. <BR>
   */
  private InstructionFactory factory = null;
  
  /** The constant pool generator. <BR>
   */
  private ConstantPoolGen constantPoolGenerator = null;
  
  /** The class generator. <BR>
   */
  private ClassGen classGenerator = null;

  /** The instrumentation configuration we use. <BR>
   */
  private InstrumentationConfiguration config = null;
  
  /** Creates a new creator for the options class. <BR>
   *  @param config the configuration values used in the static initializer
   */
  public OptionsClassCreator(InstrumentationConfiguration config) {
    this.config = config;
    classGenerator = new ClassGen("org.softevo.ample.sibrelib.runtime.Options", "java.lang.Object", "Options.java", ACC_PUBLIC | ACC_SUPER, new String[] {  });
    constantPoolGenerator = classGenerator.getConstantPool();
    factory = new InstructionFactory(classGenerator, constantPoolGenerator);
    createFields();
    createInitializer();
    createConstructor();
    createGetPropertyMethod();
    createGetOutputFileNameMethod();
    createGetIdentityMapFileNameMethod();
    createGetWindowSizeMethod();
  }

  /** Dumps the generated class into a byte array. <BR>
   *  @return the bytes of the Options class file
   *  @throws IOException this should never happen, since we dump into a byte array
   */
  public byte[] create() throws IOException {
    ByteArrayOutputStream byteBuffer;
    
    byteBuffer = new ByteArrayOutputStream();
    classGenerator.getJavaClass().dump(byteBuffer);
    return byteBuffer.toByteArray();
  }

  /** Creates the fields of the class. <BR>
   */
  private void createFields() {
    FieldGen field;

    field = new FieldGen(ACC_PUBLIC | ACC_STATIC | ACC_FINAL, Type.STRING, "KEY_OUTPUTFILENAME", constantPoolGenerator);
    field.setInitValue("outputfile");
    classGenerator.addField(field.getField());

    field = new FieldGen(ACC_PUBLIC | ACC_STATIC | ACC_FINAL, Type.STRING, "KEY_IDENTITYMAPFILENAME", constantPoolGenerator);
    field.setInitValue("identityMap");
    classGenerator.addField(field.getField());

    field = new FieldGen(ACC_PUBLIC | ACC_STATIC | ACC_FINAL, Type.STRING, "KEY_WINDOWSIZE", constantPoolGenerator);
    field.setInitValue("windowsize");
    classGenerator.addField(field.getField());

    field = new FieldGen(ACC_PRIVATE | ACC_STATIC, new ObjectType("java.util.Properties"), "properties", constantPoolGenerator);
    classGenerator.addField(field.getField());
  }

  /** Creates the static initializer of the class. <BR>
   *  This code must be altered if new attributes should be set in
   *  the options class.
   */
  private void createInitializer() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_STATIC, Type.VOID, Type.NO_ARGS, new String[] {  }, "<clinit>", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

    il.append(factory.createNew("java.util.Properties"));
    il.append(InstructionConstants.DUP);
    il.append(factory.createInvoke("java.util.Properties", "<init>", Type.VOID, Type.NO_ARGS, Constants.INVOKESPECIAL));
    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.PUTSTATIC));

    // push output file name
    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, org.softevo.ample.sibrelib.runtime.Options.KEY_OUTPUTFILENAME));
    il.append(new PUSH(constantPoolGenerator, config.getOutputFileName()));
    il.append(factory.createInvoke("java.util.Properties", "put", Type.OBJECT, new Type[] { Type.OBJECT, Type.OBJECT }, Constants.INVOKEVIRTUAL));
    il.append(InstructionConstants.POP);

    // push window size
    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, org.softevo.ample.sibrelib.runtime.Options.KEY_WINDOWSIZE));
    il.append(new PUSH(constantPoolGenerator, Integer.toString(config.getWindowSize())));
    il.append(factory.createInvoke("java.util.Properties", "put", Type.OBJECT, new Type[] { Type.OBJECT, Type.OBJECT }, Constants.INVOKEVIRTUAL));
    il.append(InstructionConstants.POP);

    // push identity map file name
    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, org.softevo.ample.sibrelib.runtime.Options.KEY_IDENTITYMAPFILENAME));
    il.append(new PUSH(constantPoolGenerator, config.getIdentityMapFileName()));
    il.append(factory.createInvoke("java.util.Properties", "put", Type.OBJECT, new Type[] { Type.OBJECT, Type.OBJECT }, Constants.INVOKEVIRTUAL));
    il.append(InstructionConstants.POP);
    
    il.append(factory.createReturn(Type.VOID));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }

  /** Creates the default constructor (required by Bytecode Verifier). <BR>
   */
  private void createConstructor() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_PUBLIC, Type.VOID, Type.NO_ARGS, new String[] {  }, "<init>", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

    il.append(factory.createLoad(Type.OBJECT, 0));
    il.append(factory.createInvoke("java.lang.Object", "<init>", Type.VOID, Type.NO_ARGS, Constants.INVOKESPECIAL));
    il.append(factory.createReturn(Type.VOID));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }

  /** Generates the <code>GetProperty</code> method. <BR>
   */
  private void createGetPropertyMethod() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_PUBLIC | ACC_STATIC, Type.STRING, new Type[] { Type.STRING }, new String[] { "arg0" }, "getProperty", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(factory.createLoad(Type.OBJECT, 0));
    il.append(factory.createInvoke("java.util.Properties", "getProperty", Type.STRING, new Type[] { Type.STRING }, Constants.INVOKEVIRTUAL));
    il.append(factory.createReturn(Type.OBJECT));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }

  /** Generates the <code>GetOutputFileName</code> method. <BR>
   */
  private void createGetOutputFileNameMethod() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_PUBLIC | ACC_STATIC, Type.STRING, Type.NO_ARGS, new String[] {  }, "getOutputFileName", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

   il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, "outputfile"));
    il.append(factory.createInvoke("java.util.Properties", "getProperty", Type.STRING, new Type[] { Type.STRING }, Constants.INVOKEVIRTUAL));
    il.append(factory.createReturn(Type.OBJECT));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }

  /** Generates the <code>GetIdentityMapFileName</code> method. <BR>
   */
  private void createGetIdentityMapFileNameMethod() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_PUBLIC | ACC_STATIC, Type.STRING, Type.NO_ARGS, new String[] {  }, "getIdentityMapFileName", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, "identityMap"));
    il.append(factory.createInvoke("java.util.Properties", "getProperty", Type.STRING, new Type[] { Type.STRING }, Constants.INVOKEVIRTUAL));
    il.append(factory.createReturn(Type.OBJECT));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }

  /** Generates the <code>GetWindowSize</code> method. <BR>
   */
  private void createGetWindowSizeMethod() {
    InstructionList il = new InstructionList();
    MethodGen method = new MethodGen(ACC_PUBLIC | ACC_STATIC, Type.INT, Type.NO_ARGS, new String[] {  }, "getWindowSize", "org.softevo.ample.sibrelib.runtime.Options", il, constantPoolGenerator);

    il.append(factory.createFieldAccess("org.softevo.ample.sibrelib.runtime.Options", "properties", new ObjectType("java.util.Properties"), Constants.GETSTATIC));
    il.append(new PUSH(constantPoolGenerator, "windowsize"));
    il.append(factory.createInvoke("java.util.Properties", "getProperty", Type.STRING, new Type[] { Type.STRING }, Constants.INVOKEVIRTUAL));
    il.append(factory.createInvoke("java.lang.Integer", "parseInt", Type.INT, new Type[] { Type.STRING }, Constants.INVOKESTATIC));
    il.append(factory.createReturn(Type.INT));
    method.setMaxStack();
    method.setMaxLocals();
    classGenerator.addMethod(method.getMethod());
    il.dispose();
  }
}
