/*
 * Created on 28.07.2004 by dallmeier
 *
 * 
 */
package org.softevo.ample.sibrelib.extensions;

import java.util.HashMap;
import org.apache.bcel.Constants;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.INVOKESPECIAL;
import org.apache.bcel.generic.Instruction;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.InvokeInstruction;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.PUSH;
import org.apache.bcel.generic.Type;
import org.softevo.sibrelib.InstrumentationContext;
import org.softevo.sibrelib.InstrumentationException;


/**
 * @author dallmeier
 *
 */
public class IncomingCallTracer extends SequenceTracer {

  /** The name of the runtime class. <BR>
   */
  protected String runtimeClassName = null;
  
  protected boolean traceInstances = false;
  
  /** Creates a new tracer instance. <BR>
   *  @param context the instrumetation context
   */
  public IncomingCallTracer(InstrumentationContext context,
                            String runtimeClassName,
                            boolean traceInstances) {
    super(context);
    this.runtimeClassName = runtimeClassName;
    this.traceInstances = traceInstances;
  }

  public void postProcessJavaClass(JavaClass javaClass,
                                   ClassGen classGenerator,
                                   ConstantPoolGen constantPoolGenerator,
                                   InstructionFactory factory) {
    Method[] methods;
    boolean finalizerPresent = false;
    MethodGen methodGenerator;
    InstructionList instructions;
    InstructionHandle[] originalInstructions;
    InvokeInstruction invokeInstruction;
    Instruction instruction;
    int instructionCounter;
    
    //let instance tracer perform modifications
    if (traceInstances) {
     instanceTracer.postProcessJavaClass(javaClass, classGenerator, 
                                         constantPoolGenerator, factory);
    }
    if (traceInstances) {
      addFinalizer(classGenerator, javaClass, constantPoolGenerator, factory, runtimeClassName);
    }
  }
  
  public InstructionList createMethodStartCode(JavaClass javaClass,
                                               ConstantPoolGen constantPoolGenerator,
                                               MethodGen methodGenerator,
                                               InstructionFactory factory) throws InstrumentationException {
    InstructionList result;
    int id;
    
    result = new InstructionList();
    result.append(factory.createInvoke(javaClass.getClassName(),
                                       ThreadTracer.METHODNAME_GETTHREADID,
                                       Type.INT, new Type[] {},
                                       Constants.INVOKESTATIC));
    if (methodGenerator.isStatic()) {
      result.append(new PUSH(constantPoolGenerator, -1));
    } else {
      if (traceInstances) {
        result.append(factory.createLoad(Type.OBJECT, 0));
        result.append(factory.createFieldAccess(javaClass.getClassName(), InstanceTracer.FIELDNAME_ID,
                                                Type.INT, Constants.GETFIELD));
      } else {
        result.append(new PUSH(constantPoolGenerator, -1));
      }
    }
    id = context.getIdentifierMap().getIdentifier(javaClass.getClassName());
    result.append(new PUSH(constantPoolGenerator, id));
    id = context.getIdentifierMap().getIdentifier(javaClass.getClassName(),
                                                  methodGenerator.getName(),
                                                  methodGenerator.getSignature());
    result.append(new PUSH(constantPoolGenerator, id));
    result.append(factory.createInvoke(runtimeClassName,
                                       "writeEntry", Type.VOID,
                                       new Type[] {Type.INT, Type.INT, Type.INT, Type.INT},
                                       Constants.INVOKESTATIC));
    return result;
  }

}
