/*
 * Created on 28.07.2004 by dallmeier
 *
 * 
 */
package org.softevo.ample.sibrelib.extensions;


import java.util.HashMap;
import org.apache.bcel.Constants;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.INVOKESPECIAL;
import org.apache.bcel.generic.Instruction;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.InvokeInstruction;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.PUSH;
import org.apache.bcel.generic.Type;
import org.softevo.sibrelib.InstrumentationContext;
import org.softevo.sibrelib.InstrumentationException;


/**
 * @author dallmeier
 *
 */
public class OutgoingCallTracer extends SequenceTracer {
  
  protected String runtimeClassName = null;
  
  protected boolean traceInstances = false;
  
  /** Creates a new tracer instance. <BR>
   *  @param context the instrumetation context
   */
  public OutgoingCallTracer(InstrumentationContext context,
                       String runtimeClassName,
                       boolean traceInstances) {
    super(context);
    this.runtimeClassName = runtimeClassName;
    this.traceInstances = traceInstances;
  }

  public void postProcessJavaClass(JavaClass javaClass,
                                   ClassGen classGenerator,
                                   ConstantPoolGen constantPoolGenerator,
                                   InstructionFactory factory) {
    Method[] methods;
    boolean finalizerPresent = false;
    MethodGen methodGenerator;
    InstructionList instructions;
    InstructionHandle[] originalInstructions;
    InvokeInstruction invokeInstruction;
    Instruction instruction;
    int instructionCounter;

    if (traceInstances) {
      //let instance tracer perform modifications
      instanceTracer.postProcessJavaClass(javaClass, classGenerator, 
                                          constantPoolGenerator, factory);
    }
    methods = classGenerator.getMethods();
    for(int counter = 0; counter < methods.length; counter++) {
      methodGenerator = new MethodGen(methods[counter], javaClass.getClassName(), constantPoolGenerator);
      if (!methods[counter].isAbstract() && !methods[counter].isNative() && 
          threadTracer.allowTraceInstrumentation(methodGenerator) &&
          instanceTracer.allowTraceInstrumentation(methodGenerator)) {
        instructionCounter = 0;
        instructions = methodGenerator.getInstructionList();
        originalInstructions = instructions.getInstructionHandles();
        if (methodGenerator.getName().equals("<init>")) {
          for (; instructionCounter < originalInstructions.length; instructionCounter++) {
            instruction = originalInstructions[instructionCounter].getInstruction();
            if (instruction instanceof INVOKESPECIAL) {
              invokeInstruction = (INVOKESPECIAL) instruction;
              //we found the super class constructor call
              if (invokeInstruction.getClassName(constantPoolGenerator).equals(javaClass.getSuperclassName())) {
                instructionCounter++;
                break;
              }
            }
          }
        }
        for (int counter2 = instructionCounter; counter2 < originalInstructions.length; counter2++) {
          if (originalInstructions[counter2].getInstruction() instanceof InvokeInstruction) {
            invokeInstruction = (InvokeInstruction) originalInstructions[counter2].getInstruction();
            if (!invokeInstruction.getClassName(constantPoolGenerator).startsWith("org.softevo.ample.sibrelib.runtime") && 
                (!invokeInstruction.getMethodName(constantPoolGenerator).equals(ThreadTracer.METHODNAME_GETTHREADID)) &&
                !context.isCoreClass(invokeInstruction.getClassName(constantPoolGenerator))) {
              instructions.insert(originalInstructions[counter2], getStoreCallerInstructions(javaClass, methodGenerator, 
                                                                                             constantPoolGenerator, factory));
            }
          }
        }
        methodGenerator.setMaxLocals();
        methodGenerator.setMaxStack();
        classGenerator.setMethodAt(methodGenerator.getMethod(), counter);
        //System.out.println(classGenerator.getClassName() + "#" + methodGenerator.getName());
        //System.out.println(methodGenerator.getMethod().getCode());
      }
    }
    if (traceInstances) {
      addFinalizer(classGenerator, javaClass, constantPoolGenerator, factory, runtimeClassName);
    }
  }
  

  
  public InstructionList createMethodStartCode(JavaClass javaClass,
                                               ConstantPoolGen constantPoolGenerator,
                                               MethodGen methodGenerator,
                                               InstructionFactory factory) throws InstrumentationException {
    InstructionList result;
    int id;
    
    result = new InstructionList();
    /*result.append(factory.createPrintln(javaClass.getClassName() + "#" + 
                                        methodGenerator.getName()));*/
    result.append(factory.createInvoke(javaClass.getClassName(),
                                       ThreadTracer.METHODNAME_GETTHREADID,
                                       Type.INT, new Type[] {},
                                       Constants.INVOKESTATIC));
    
    id = context.getIdentifierMap().getIdentifier(javaClass.getClassName());
    result.append(new PUSH(constantPoolGenerator, id));
    id = context.getIdentifierMap().getIdentifier(javaClass.getClassName(),
                                                  methodGenerator.getName(),
                                                  methodGenerator.getSignature());
    result.append(new PUSH(constantPoolGenerator, id));
    result.append(factory.createInvoke(runtimeClassName,
                                       "writeEntry", Type.VOID,
                                       new Type[] {Type.INT, Type.INT, Type.INT},
                                       Constants.INVOKESTATIC));   
    return result;
  }

  protected InstructionList getStoreCallerInstructions(JavaClass javaClass,
                                                     MethodGen methodGenerator,
                                                     ConstantPoolGen constantPoolGenerator,
                                                     InstructionFactory factory) {
    InstructionList result;
    int id;
    
    result = new InstructionList();
    result.append(factory.createInvoke(javaClass.getClassName(),
                                       ThreadTracer.METHODNAME_GETTHREADID,
                                       Type.INT, new Type[] {},
                                       Constants.INVOKESTATIC));
    if (!traceInstances || methodGenerator.isStatic()) {
      result.append(new PUSH(constantPoolGenerator, -1));
    } else {
      result.append(factory.createLoad(Type.OBJECT, 0));
      result.append(factory.createFieldAccess(javaClass.getClassName(), InstanceTracer.FIELDNAME_ID,
                                              Type.INT, Constants.GETFIELD));
    }
    id = context.getIdentifierMap().getIdentifier(javaClass.getClassName());
    result.append(new PUSH(constantPoolGenerator, id));
    result.append(factory.createInvoke(runtimeClassName,
                                       "setCaller", Type.VOID,
                                       new Type[] {Type.INT, Type.INT, Type.INT},
                                       Constants.INVOKESTATIC));   
    return result;
  }
  
}
