/*
 *
 */
package org.softevo.ample.sibrelib.runtime;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.OutputStreamWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.apache.commons.lang.IntHashMap;
import org.softevo.sibrelib.runtime.CalledMethodsTracer;
import org.softevo.sibrelib.runtime.ConfigFileReader;
import org.softevo.sibrelib.runtime.Constants;
import org.softevo.sibrelib.runtime.util.Utilities;

/**
 * @author dallmeier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class SequenceWriter implements Constants {

  protected static IntHashMap classToPatternSetMap = null;
  
  protected static int windowSize = 5;
  
  protected static Object lock = new Object();

  protected static int entryCounter = 0;
  
  protected static String outputFileName = "patterns.xml";
  
  protected static String identityMapFileName = "identity.map";
  
  protected static HashMap identityMap = null;
  
  protected static CalledMethodsTracer methodTracer = null;

  protected static void readConfiguration() {
    System.out.println("Reading config.");
    outputFileName = Options.getOutputFileName();
    identityMapFileName = Options.getIdentityMapFileName();
    windowSize = Options.getWindowSize();
    readIdentityMap(identityMapFileName);
    methodTracer = new CalledMethodsTracer(identityMap);
  }
  
  protected static void readIdentityMap(String fileName) {
    FileInputStream fileInputStream = null;
    ObjectInputStream objectInputStream;
    
    try {
      fileInputStream = new FileInputStream(identityMapFileName);
      objectInputStream = new ObjectInputStream(fileInputStream);
      identityMap = (HashMap) objectInputStream.readObject(); 
    } catch (Exception ioException) {
      throw new RuntimeException("Unable to read identity map: " + ioException);
    }
  }

  protected static void writeResults(IntHashMap classToPatternSetMap, PatternNameWriter patternNameWriter) {
    int[] keys;
    HashMap patternMap;
    Iterator scanner;
    Pattern pattern;
    Object key;
    FileOutputStream resultFile = null;
    OutputStreamWriter writer;
    HashMap reverseMap;
    Object value;
    String className;
    Vector sortedKeys;
    File lockFile;
    
    synchronized (lock) {
      reverseMap = new HashMap(identityMap.size());
      scanner = identityMap.keySet().iterator();
      while (scanner.hasNext()) {
        key = scanner.next();
        value = identityMap.get(key);
        reverseMap.put(value, key);
      }
      try {
        resultFile = new FileOutputStream(outputFileName);
        writer = new OutputStreamWriter(resultFile);
        writer.write("<?xml version=\"1.0\"?>\n<patterns traceSize=\"" + 
                     entryCounter + "\" windowSize=\"" + windowSize + "\" classCount=\""
                     + classToPatternSetMap.size() + "\">\n");
        writer.flush();
        methodTracer.writeSymbolsAsXML(resultFile, reverseMap);
        keys = classToPatternSetMap.getKeys();
        sortedKeys = new Vector(keys.length);
        for (int counter = 0; counter < keys.length; counter++) {
          sortedKeys.add(new Integer(keys[counter]));
        }
        Collections.sort(sortedKeys);
        /*System.out.println("-------------------------");
        for (int counter = 0; counter < sortedKeys.size(); counter++) {
          System.out.println(sortedKeys.get(counter));
        }
        System.out.println(classToPatternSetMap + " Keys: " + keys.length);*/
        for (int counter = 0; counter < sortedKeys.size(); counter++) {
          className = reverseMap.get(sortedKeys.get(counter)).toString();
          writer.write("\t<class name=\"" + className +
          "\">\n");
          patternMap = (HashMap) classToPatternSetMap.get(keys[counter]);
          scanner = patternMap.keySet().iterator();
          while (scanner.hasNext()) {
            pattern = (Pattern) scanner.next();
            writer.write("\t\t<pattern occurences=\"" + pattern.occurences + "\" discoveryIndex=\"" + pattern.discoveryIndex + 
                         "\" lastOccurence=\"" + pattern.lastOccurence + "\">\n");
            for (int counter2 = 0; counter2 < pattern.pattern.length; counter2++) {
              try {
                writer.write("\t\t\t<method name=\"" + Utilities.escape(patternNameWriter.getEntityName(reverseMap.get(new Integer(pattern.pattern[counter2])).toString(), className)) +
                "\"/>\n");
              } catch (StringIndexOutOfBoundsException oops) {
                System.out.println("Unable to resolve correct class name for id " + pattern.pattern[counter2] + ". Wrong identity map?");
                System.out.println("Halting program execution.");
                System.exit(0);
              }
            }
            writer.write("\t\t</pattern>\n");
          }
          writer.write("\t</class>\n");
        }
        writer.write("</patterns>");
        writer.flush();
      } catch (IOException ioException) {
        System.out.println("An exception occured writing pattern file: " + ioException);
      } finally {
        if (resultFile != null) {
          try {
            resultFile.close();
          } catch (IOException finalizeException) {
            //ignore exception
          }
        }
        lockFile = new File(".lock");
        try {
          lockFile.createNewFile();
        } catch (IOException exception) {
          //pray we never regret this empty block
        }
      }
    }
  }
  
  /** Gets the pattern set  for a class. <BR>
   *  If none exists in the map, a new instance
   *  is created.
   */
  protected static HashMap getOrCreatePatternSet(int classId) {
    HashMap result;
    
    result = (HashMap) classToPatternSetMap.get(classId);
    if (result == null) {
      result = new HashMap(100);
      classToPatternSetMap.put(classId, result);
    }
    return result;
  }  
  
  protected abstract static class PatternNameWriter {
    
    public abstract String getEntityName(String mapContent, String className);
    
  }

  protected static class Pattern {
    
    int[] pattern = null;
  
    int occurences = 1;
    
    int discoveryIndex = 0;
    
    int lastOccurence = 0;
    
    public int hashCode() {
      int result;
      
      result = pattern[0];
      for (int counter = 1; counter < pattern.length; counter++) {
        result ^= pattern[counter];
      }
      return result;
    }
    
    public boolean equals(Object object) {
      Pattern candidate;
  
      candidate = (Pattern) object;
      if (candidate.pattern.length != pattern.length) {
        return false;
      }
      for (int counter = 0; counter < pattern.length; counter++) {
        if (pattern[counter] != candidate.pattern[counter]) {
          return false;
        }
      }
      return true;
    }
    
    public String toString() {
      StringBuffer result;
      
      result = new StringBuffer();
      for (int counter = 0; counter < pattern.length; counter++) {
        result.append(pattern[counter]);
        result.append(',');
      }
      return result.toString();
    }
  }


}
