/*
 *
 */
package org.softevo.ample.ui;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.softevo.ample.AmpleMessages;
import org.softevo.ample.AmplePlugin;

public class AmplePreferencePage extends PreferencePage implements IAmplePreferenceConstants, IWorkbenchPreferencePage {

  private Combo windowSizeCombo = null;
  
  private Combo traceLevelCombo = null;
  
  private Combo perspectiveCombo = null;

  private Combo durationCombo = null;
  
  
  public AmplePreferencePage() {
    super();
  }

  public AmplePreferencePage(String title) {
    super(title);
  }

  public AmplePreferencePage(String title, ImageDescriptor image) {
    super(title, image);
  }

  protected Control createContents(Composite parent) {
    Composite umbrella;
    
    getStore().setDefault(KEY_WINDOWSIZE, 5);
    getStore().setDefault(KEY_LEVEL, 0);
    getStore().setDefault(KEY_PERSPECTIVE, 0);
    getStore().setDefault(KEY_TESTDURATION, 0);
    umbrella = createUmbrella(parent);
    createCombos(umbrella);
    initializeValues();
    return umbrella;
  }
  
  protected void createAllCombos(Composite parent) {
    Composite composite;
    GridLayout layout;
    Label label;
    
    composite = new Composite(parent, SWT.NONE);
    layout = new GridLayout(2, false);
    composite.setLayout(layout);
    label = new Label(composite, SWT.NONE);
    label.setText(AmpleMessages.Pref_windowsize_label);
    windowSizeCombo = new Combo(composite, SWT.READ_ONLY);
    windowSizeCombo.setItems(PREF_WINDOWSIZES);
    label = new Label(composite, SWT.NONE);
    label.setText(AmpleMessages.Pref_level_label);
    traceLevelCombo = new Combo(composite, SWT.READ_ONLY);
    traceLevelCombo.setItems(PREF_LEVELS);
    label = new Label(composite, SWT.NONE);
    label.setText(AmpleMessages.Pref_perspective_label);
    perspectiveCombo = new Combo(composite, SWT.READ_ONLY);
    perspectiveCombo.setItems(PREF_PERSPECTIVES);
    label = new Label(composite, SWT.NONE);
    label.setText(AmpleMessages.Pref_duration_label);
    durationCombo = new Combo(composite, SWT.READ_ONLY);
    durationCombo.setItems(PREF_DURATION);
    
  }
  
  protected void createCombos(Composite parent) {
    Font mainFont;
    Composite textComposite;
    GridLayout textLayout;
    
    mainFont = parent.getFont();
    textComposite = new Composite(parent, SWT.NONE);
    textComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    textLayout = new GridLayout();
    textLayout.marginWidth = 0;
    textLayout.marginHeight = 20;
    textLayout.verticalSpacing = 10;
    textComposite.setLayout(textLayout);
    textComposite.setFont(mainFont);

    Label descriptionLabel = new Label(textComposite, SWT.NONE | SWT.WRAP);
    descriptionLabel.setText(AmpleMessages.Pref_heading); //$NON-NLS-1$
    descriptionLabel.setFont(mainFont);
    
    createAllCombos(textComposite);
  }

  private void initializeValues() {
    int windowSize, levelPosition, perspectivePosition, durationPosition;
    
    windowSize = getStore().getInt(KEY_WINDOWSIZE);
    windowSizeCombo.select(windowSize);
    
    levelPosition = getStore().getInt(KEY_LEVEL);
    if (levelPosition != -1) {
      traceLevelCombo.select(levelPosition);
    }

    perspectivePosition = getStore().getInt(KEY_PERSPECTIVE);
    if (perspectivePosition != -1) {
      perspectiveCombo.select(perspectivePosition);
    }
    durationPosition = getStore().getInt(KEY_TESTDURATION);
    if (durationPosition != -1) {
      durationCombo.select(durationPosition);
    }

  }
  
  protected Composite createUmbrella(Composite parent) {
    Composite result;
    Font font;
    
    font = parent.getFont();
    result = new Composite(parent, SWT.NONE);
    result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    result.setFont(font);

    GridLayout layout = new GridLayout();
    layout.marginWidth = 0;
    layout.marginHeight = 0;
    layout.verticalSpacing = 10;
    result.setLayout(layout);

    return result;
  }

  protected IPreferenceStore getStore() {
    return AmplePlugin.getDefault().getPreferenceStore();
  }
  
  public boolean performOk() {
    if (super.performOk()) {
      if (windowSizeCombo.getSelectionIndex() != -1) {
        getStore().setValue(KEY_WINDOWSIZE, windowSizeCombo.getSelectionIndex());
      }
      if (traceLevelCombo.getSelectionIndex() != -1) {
        getStore().setValue(KEY_LEVEL, traceLevelCombo.getSelectionIndex());
      }
      if (perspectiveCombo.getSelectionIndex() != -1) {
        getStore().setValue(KEY_PERSPECTIVE, perspectiveCombo.getSelectionIndex());
      }
      if (durationCombo.getSelectionIndex() != -1) {
        getStore().setValue(KEY_TESTDURATION, durationCombo.getSelectionIndex());
      }
      return true;
    } else {
      return false;
    }
  }
  
  protected void performDefaults() {
    getStore().setToDefault(KEY_WINDOWSIZE);
    getStore().setToDefault(KEY_LEVEL);
    getStore().setToDefault(KEY_PERSPECTIVE);
    getStore().setToDefault(KEY_TESTDURATION);
    initializeValues();
  }
  
  public void init(IWorkbench workbench) {
    // TODO Auto-generated method stub
    
  }
  
}
