/*
 *
 */
package org.softevo.ample.ui;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.softevo.ample.eval.PatternAndWeight;
import org.softevo.ample.eval.PatternDifferences;
import org.softevo.ample.instrument.InstrumentationConfiguration;

public class PatternDifferencesDialog extends Dialog {
  
  private PatternDifferences differences = null;
  
  private InstrumentationConfiguration config = null;
  
  private TableViewer tableViewer = null;
  
  public PatternDifferencesDialog(Shell parentShell, PatternDifferences differences, InstrumentationConfiguration config) {
    super(parentShell);
    this.differences = differences;
    this.config = config;
  }

  protected void configureShell(Shell shell) {
    super.configureShell(shell);
    shell.setText("Patterns for class " + differences.getClassName());
  }
  
  protected Control createDialogArea(Composite parent) {
    PatternAndWeight[] patterns;
    int style = SWT.H_SCROLL | SWT.V_SCROLL;
    GridLayout layout;
    
    layout = new GridLayout(1, false);
    layout.marginHeight = 20;
    parent.setLayout(layout);
    GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
    gridData.horizontalSpan = 1;
    gridData.widthHint = 600;
    gridData.heightHint = 500;
    parent.setLayoutData(gridData);
    
    if (getButton(IDialogConstants.CANCEL_ID) != null) {
      getButton(IDialogConstants.CANCEL_ID).setEnabled(false);
    }
    Table table = new Table(parent, style);
    gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
    gridData.horizontalSpan = 1;
    table.setLayoutData(gridData);

    table.setLinesVisible(false);
    table.setHeaderVisible(true);

    TableColumn column = new TableColumn(table, SWT.CENTER, 0);
    column.setText("Passing Runs");
    column.setWidth(100);

    column = new TableColumn(table, SWT.CENTER, 1);
    column.setText("Failing Runs");
    column.setWidth(100);
    
    column = new TableColumn(table, SWT.CENTER, 2);
    column.setText("Sequence Weight");
    column.setWidth(120);
    column.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        PatternSorter sorter;
        
        sorter = (PatternSorter) tableViewer.getSorter();
        tableViewer.setSorter(new PatternSorter(!sorter.getAscending()));
      }
    });
    
    
    column = new TableColumn(table, SWT.LEFT, 3);
    column.setText("Pattern");
    column.setWidth(800);
    
    tableViewer = new TableViewer(table);
    patterns = (PatternAndWeight[]) differences.getPatternDifferences().toArray(new PatternAndWeight[differences.getPatternDifferences().size()]);
    tableViewer.setContentProvider(new PatternDifferencesContentProvider(patterns));
    tableViewer.setLabelProvider(new PatternDifferencesLabelProvider(config));
    tableViewer.setSorter(new PatternSorter(false));
    tableViewer.setInput(patterns);
    
    return parent;
  }  
}
