package org.softevo.ample.ui;

import java.text.MessageFormat;
import java.util.ArrayList;

import org.eclipse.jdt.internal.junit.ui.TestRunInfo;
import org.eclipse.jdt.internal.junit.ui.TestRunnerViewPart;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.softevo.ample.AmpleMessages;
import org.softevo.ample.AmplePlugin;
import org.softevo.ample.AmpleSettings;

/**
 * Dialog to choose tests on which Ample is to be executed.
 */
public class SelectTestsDialog extends Dialog {

	/**
	 * Initializes a new instance of <code>SelectTestsDialog</code>.
	 * 
	 * @param parentShell
	 *            Owner of this dialog window.
	 * @param failingTestIndex
	 *            Index of failing test initially selected.
	 */
	public SelectTestsDialog(Shell parentShell, int failingTestIndex) {
		super(parentShell);
		passingTestsList = null;
		failingTestsList = null;
		selectAllPassingCheck = null;
		selectedFailingTest = (TestRunInfo) AmplePlugin.getSharedListener().getFailingTest(failingTestIndex);
		selectedPassingTests = null;
		failingTestIcon = TestRunnerViewPart.createImage("obj16/testfail.gif");
		passingTestIcon = TestRunnerViewPart.createImage("obj16/testok.gif");
	}

	/**
	 * Gets the selected failing test.
	 * <p>This method must be called only after the user has pressed the OK button of this dialog.</p>
	 * 
	 * @return <code>TestRunInfo</code> corresponding to the failing test the user has selected.
	 */
	public TestRunInfo getSelectedFailingTest() {
		return selectedFailingTest;
	}

	/**
	 * Gets the selected passing tests.
	 * <p>This method must be called only after the user has pressed the OK button of this dialog.</p>
	 * 
	 * @return Array of <code>TestRunInfo</code>s corresponding to the passing tests the user has
	 *         selected.
	 */
	public TestRunInfo[] getSelectedPassingTests() {
		return selectedPassingTests;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.windows.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(AmpleMessages.SelectTestsDialog_title);
	}

	/* (non-Javadoc)
	 * @see Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);

		// Create the list of failing tests
		ArrayList failingTests = AmplePlugin.getSharedListener().getFailingTests();
		Label failingTestsLabel = createLabelWithIcon(composite, failingTestIcon);

		if (failingTests.size() > 1) {
			failingTestsLabel.setText(AmpleMessages.SelectTestsDialog_select_failing_test);

			// Display a list of failing tests to choose from
			failingTestsList = new List(composite, SWT.BORDER | SWT.SINGLE);
			fillListWithTests(failingTestsList, failingTests, selectedFailingTest);

		} else { // failingTests.size() == 1

			// Create a label informing which test fails
			failingTestsLabel.setText(AmpleMessages.SelectTestsDialog_failing_test);
			selectedFailingTest = (TestRunInfo) failingTests.get(0);
			String testName = createTestName(selectedFailingTest);
			Label failingTestNameLabel = new Label(composite, SWT.LEFT);
			failingTestNameLabel.setText(testName);
		}

		// Create a horizontal separator
		Label separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		separator.setText("");

		// Create a list of passing tests
		ArrayList passingTests = AmplePlugin.getDefault().getListener().getPassingTests();
		Label passingTestsLabel = createLabelWithIcon(composite, passingTestIcon);

		if (passingTests.size() > AmpleSettings.MIN_PASSING_TESTS) {

			if (AmpleSettings.MIN_PASSING_TESTS == 1) {
				passingTestsLabel.setText(AmpleMessages.SelectTestsDialog_select_passing_test);
			} else {
				String message = MessageFormat.format(AmpleMessages.SelectTestsDialog_select_passing_tests,
				                                      new String[] { String.valueOf(AmpleSettings.MIN_PASSING_TESTS) });
				passingTestsLabel.setText(message);
			}

			// Display a list of failing tests to choose from
			passingTestsList = new List(composite, SWT.BORDER | SWT.MULTI);
			fillListWithTests(passingTestsList, passingTests, null);
			passingTestsList.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
				}
				public void widgetSelected(SelectionEvent e) {
					updateSelectionPassing();
				}
			});

			selectAllPassingCheck = new Button(composite, SWT.CHECK | SWT.LEFT);
			selectAllPassingCheck.setSelection(false);
			selectAllPassingCheck.setText(AmpleMessages.SelectTestsDialog_select_all_passing);
			selectAllPassingCheck.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
				}

				public void widgetSelected(SelectionEvent e) {
					allPassingClicked();
				}
			});

		} else { // passingTests.size() == AmpleSettings.MIN_PASSING_TESTS

			// Create a label informing which test(s) pass
			if (AmpleSettings.MIN_PASSING_TESTS == 1) {
				passingTestsLabel.setText(AmpleMessages.SelectTestsDialog_passing_test);
			} else {
				passingTestsLabel.setText(AmpleMessages.SelectTestsDialog_passing_tests);
			}
			// TODO: Check if the following initialization of selectedPassingTests works
			selectedPassingTests = (TestRunInfo[]) AmplePlugin.getSharedListener().getPassingTests().toArray(new TestRunInfo[AmplePlugin.getSharedListener().getPassingTests().size()]);
			StringBuffer testNames = new StringBuffer();
			for (int i = 0; i < passingTests.size(); ++i) {
				TestRunInfo tInfo = (TestRunInfo) passingTests.get(i);
				testNames.append("\n");
				testNames.append(createTestName(tInfo));
			}
			Label passingTestNamesLabel = new Label(composite, SWT.LEFT);
			passingTestNamesLabel.setText(testNames.toString());
		}

		RowLayout compositeLayout = new RowLayout();
		compositeLayout.type = SWT.VERTICAL;
		compositeLayout.fill = true;
		compositeLayout.spacing = SPACING;
		compositeLayout.marginTop = MARGINS;
		compositeLayout.marginBottom = MARGINS;
		compositeLayout.marginLeft = MARGINS;
		compositeLayout.marginRight = MARGINS;
		composite.setLayout(compositeLayout);
		return composite;
	}

	/**
	 * Creates a new button with a given ID.
	 * <p>
	 * This method overrides {@link Dialog#createButton(org.eclipse.swt.widgets.Composite, int, java.lang.String, boolean)}
	 * by additionally disabling the OK button if the user must choose passing tests before pressing it.
	 * {@inheritDoc}
	 */
	protected Button createButton(Composite parent, int id, String label, boolean defaultButton) {
		Button button = super.createButton(parent, id, label, defaultButton);
		if (id == IDialogConstants.OK_ID && passingTestsList != null) {
			button.setEnabled(false);
		}
		return button;
	}

	/**
     * Notifies that the OK button of this dialog has been pressed.
	 * {@inheritDoc}
	 */
	protected void okPressed() {
		if (failingTestsList != null) {
			int index = failingTestsList.getSelectionIndex();
			selectedFailingTest = AmplePlugin.getSharedListener().getFailingTest(index);
		}
		if (passingTestsList != null) {
			int[] indices = passingTestsList.getSelectionIndices();
			selectedPassingTests = AmplePlugin.getSharedListener().getPassingTests(indices);
		}
		super.okPressed();
	}

	/**
	 * Creates a composite control that contains a label with an icon on its left.
	 *
	 * @param dialogArea Parent control of the newly creted composite.
	 * @param icon Icon to be displayed to the left of the label.
	 * @return Newly created composite's <code>Label</code>.
	 */
	private static Label createLabelWithIcon(Composite dialogArea, Image icon) {
		Composite titleComposite = new Composite(dialogArea, SWT.EMBEDDED);
		Label iconLabel = new Label(titleComposite, SWT.LEFT);
		iconLabel.setImage(icon);
		Label textLabel = new Label(titleComposite, SWT.LEFT);
		RowLayout layout = new RowLayout();
		layout.type = SWT.HORIZONTAL;
		layout.fill = true;
		layout.marginLeft = layout.marginRight = 0;
		layout.marginBottom = layout.marginTop = 0;
		layout.spacing = SPACING;
		titleComposite.setLayout(layout);
		return textLabel;
	}

	/**
	 * Creates a single-line description of a test run to be displayed to the user.
	 * 
	 * @param testInfo
	 *            Test run info that will be shown.
	 * @return <code>String</code> representation of the given test info.
	 */
	private static String createTestName(TestRunInfo testInfo) {
		return testInfo.getTestMethodName() + " - " + testInfo.getClassName();
		// return testInfo.getTestId() + " " + testInfo.getTestName();
	}

	/**
	 * Updates the dialog when the user changes the selection of passing tests.
	 * <p>This method makes sure the dialog's OK button is enabled only if enough passing tests are
	 * selected by the user.</p>
	 */
	private void updateSelectionPassing() {
		int selectedCount = passingTestsList.getSelectionCount();
		if (selectedCount < AmpleSettings.MIN_PASSING_TESTS) {
			getButton(IDialogConstants.OK_ID).setEnabled(false);
			selectAllPassingCheck.setSelection(false);
		} else {
			getButton(IDialogConstants.OK_ID).setEnabled(true);
			if (selectedCount == passingTestsList.getItemCount()) {
				selectAllPassingCheck.setSelection(true);
			} else {
				selectAllPassingCheck.setSelection(false);
			}
		}
	}

	/**
	 * Fills a given list widget with a the names of a specified list of tests.
	 * 
	 * @param aTestList
	 *            List widget to be filled with test names.
	 * @param aTestInfos
	 *            List of tests which names are to be added to the list.
	 * @param selectedTest
	 *            Test that will be initially selected. Set this parameter to <code>null</code> if
	 *            no test is to be initially selected.
	 */
	private void fillListWithTests(List aTestList, ArrayList aTestInfos, TestRunInfo selectedTest) {
		for (int i = 0; i < aTestInfos.size(); ++i) {
			TestRunInfo tInfo = (TestRunInfo) aTestInfos.get(i);
			aTestList.add(createTestName(tInfo));
			if (selectedTest != null && tInfo.getTestId().equals(selectedTest.getTestId())) {
				aTestList.select(i);
			}
		}
	}

	/**
	 * Updates the dialog when the checkbox for selecting all passing tests is clicked on.
	 * <p>This method selects / deselects all items in the list of the passing tests and updates the
	 * &quot;OK&quot; button of the dialog accordingly.</p>
	 */
	private void allPassingClicked() {
		if (selectAllPassingCheck.getSelection()) {
			passingTestsList.selectAll();
			getButton(IDialogConstants.OK_ID).setEnabled(true);
		} else {
			passingTestsList.deselectAll();
			getButton(IDialogConstants.OK_ID).setEnabled(false);
		}
	}

	/**
	 * Spacing, in pixels, between controls in the dialog.
	 */
	private static final int SPACING = 8;

	/**
	 * Margins, in pixels, between the controls and the border of the dialog.
	 */
	private static final int MARGINS = 10;

	/**
	 * List control that shows the names of the failing tests.
	 * <p>Note that such a control might not be created.</p>
	 */
	private List failingTestsList;

	/**
	 * List control that shows the names of the passing tests.
	 * <p>Note that such a control might not be created.</p>
	 */
	private List passingTestsList;

	/**
	 * Checkbox used to select all passing tests.
	 * <p>Note that this control is created only when {@link #passingTestsList} is created.</p>
	 * 
	 * @see #passingTestsList
	 */
	private Button selectAllPassingCheck;

	/**
	 * Icon displayed to the left of the &quot;failing test&quot; label.
	 */
	private Image failingTestIcon;

	/**
	 * Icon displayed to the left of the &quot;passing tests&quot; label.
	 */
	private Image passingTestIcon;

	/**
	 * Id of the failing test selected by the user for instrumentation.
	 */
	private TestRunInfo selectedFailingTest;

	/**
	 * List of passing runs selected by the user.
	 */
	private TestRunInfo[] selectedPassingTests;
}
