/*
 *
 */
package org.softevo.ample.ui.views;

import java.util.HashMap;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.softevo.ample.AmpleMessages;
import org.softevo.ample.AmplePlugin;
import org.softevo.ample.eval.ClassNameWithWeight;
import org.softevo.ample.eval.PatternDifferences;
import org.softevo.ample.instrument.InstrumentationConfiguration;
import org.softevo.ample.ui.PatternDifferencesDialog;

public class RankingView extends ViewPart implements IDoubleClickListener {

  private TableViewer tableViewer = null;
  
  private HashMap classNameToJavaTypeMap = null;
  
  private HashMap classNameToPatternDifferencesMap = null;
  
  private InstrumentationConfiguration config = null;
  
  public RankingView() {
    super();
    AmplePlugin.setRankingView(this);
  }

  public void createPartControl(Composite parent) {
    int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL |
    SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

    Table table = new Table(parent, style);

    GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
    gridData.grabExcessVerticalSpace = true;
    gridData.horizontalSpan = 1;
    table.setLayoutData(gridData);

    table.setLinesVisible(false);
    table.setHeaderVisible(true);

    // Entity Name (Element) Column:
    TableColumn column = new TableColumn(table, SWT.LEFT, 0);
    column.setText(AmpleMessages.RankingView_column_class);
    column.setWidth(450);
    // Add listener to column for sorting:
    column.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
              RankingSorter sorter;
              
              sorter = (RankingSorter) tableViewer.getSorter();
              tableViewer.setSorter(new RankingSorter(!sorter.getAscending(), RankingSorter.MODE_CLASSNAME));
            }
    });

    // Risk Value (Risk) Column:
    column = new TableColumn(table, SWT.LEFT, 1);
    column.setText(AmpleMessages.RankingView_column_weight);
    column.setWidth(500);
    // Add listener to column for sorting:
    column.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
              RankingSorter sorter;
              
              sorter = (RankingSorter) tableViewer.getSorter();
              tableViewer.setSorter(new RankingSorter(!sorter.getAscending(), RankingSorter.MODE_WEIGHT));
            }
    });
    
    tableViewer = new TableViewer(table);
    tableViewer.setContentProvider(new RankingContentProvider());
    tableViewer.setLabelProvider(new RankingLabelProvider());
    tableViewer.setSorter(new RankingSorter(false, RankingSorter.MODE_WEIGHT));
    
    tableViewer.addDoubleClickListener(this);
    table.addMouseListener(new MouseListener() {

      public void mouseDoubleClick(MouseEvent e) {
      }

      public void mouseDown(MouseEvent e) {
      }

      public void mouseUp(MouseEvent event) {
        ClassNameWithWeight entry;
        PatternDifferences patternDifferences;
        PatternDifferencesDialog dialog;

        if (event.button == 3 || (event.button == 1 && ((event.stateMask & SWT.CTRL) > 0))) {
          entry = (ClassNameWithWeight) (((IStructuredSelection)tableViewer.getSelection()).getFirstElement());
          if (entry != null) {
            if (classNameToPatternDifferencesMap.containsKey(entry.getClassName())) {
              patternDifferences = (PatternDifferences) classNameToPatternDifferencesMap.get(entry.getClassName());
              System.out.println(patternDifferences.getPatternDifferences().size());
              dialog = new PatternDifferencesDialog(getViewSite().getShell(), patternDifferences, config);
              dialog.open();
            }
          }
        }
      }
      
    });
  }

  public void setFocus() {
    tableViewer.getControl().setFocus();
  }

  public void doubleClick(DoubleClickEvent event) {
    ClassNameWithWeight entry;
    IType javaType;
    IEditorPart javaEditor;
    
    entry = (ClassNameWithWeight) (((IStructuredSelection)event.getSelection()).getFirstElement());
    if ((classNameToJavaTypeMap != null) && (classNameToJavaTypeMap.containsKey(entry.getClassName()))) {
      javaType = (IType) classNameToJavaTypeMap.get(entry.getClassName());
      try {
        javaEditor = JavaUI.openInEditor(javaType);
        JavaUI.revealInEditor(javaEditor, (IJavaElement) javaType);
      } catch (PartInitException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      } catch (JavaModelException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
  }

  public void showRunning() {
    tableViewer.setLabelProvider(new ITableLabelProvider() {

      public Image getColumnImage(Object element, int columnIndex) {
        // TODO Auto-generated method stub
        return null;
      }

      public String getColumnText(Object element, int columnIndex) {
        if (columnIndex == 0) {
          return AmpleMessages.RankingView_busy;
        } else if (columnIndex == 1) {
          return "...";
        } else {
          return null;
        }
      }

      public void addListener(ILabelProviderListener listener) {
      }

      public void dispose() {
      }

      public boolean isLabelProperty(Object element, String property) {
        return false;
      }

      public void removeListener(ILabelProviderListener listener) {
      }
      
    });
    tableViewer.setInput(new ClassNameWithWeight[] {new ClassNameWithWeight("", 0)} );
  }
  
  public void showBlank() {
    tableViewer.setLabelProvider(new RankingLabelProvider());
    tableViewer.setInput(new ClassNameWithWeight[0]);
  }

  public void setInput(ClassNameWithWeight[] ranking, HashMap classNameToJavaTypeMap, 
                       HashMap classNameToPatternDifferencesMap, InstrumentationConfiguration config) {
    tableViewer.setLabelProvider(new RankingLabelProvider());
    tableViewer.setInput(ranking);
    this.classNameToJavaTypeMap = classNameToJavaTypeMap;
    this.classNameToPatternDifferencesMap = classNameToPatternDifferencesMap;
    this.config = config;
  }
}
